
/*
 * CONTROL.C	DSP simulation control
 */


#include "control.h"
#include "memory.h"
#include "view.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <setjmp.h>


FILE *ctrl_out = NULL;
FILE *data_out = NULL;

static jmp_buf restart;


void error(char *msg)
{
  printf("\n**** ERROR: %s ****\n", msg);
  yycleanup();
  if (ctrl_out != NULL) {
    fclose(ctrl_out);
    ctrl_out = NULL;
  }
  if (data_out != NULL) {
    fclose(data_out);
    data_out = NULL;
  }
  deallocate_all();
  longjmp(restart, 1);
}


char bin_dir[200];
char lib_dir[200];


void set_directories(char *bin_name)
{
  char *cp;

  strcpy(bin_dir, bin_name);
  cp = bin_dir;
  while (*cp != '\0') {
    cp++;
  }
#ifdef WINDOWS
  while (*cp != '\\' && cp != bin_dir) {
    cp--;
  }
  if (cp == bin_dir) {
    strcpy(bin_dir, ".\\");
  } else {
    *++cp = '\0';
  }
  strcpy(lib_dir, bin_dir);
  strcat(lib_dir, "..\\lib\\");
#endif /* WINDOWS */
#ifdef UNIX
  while (*cp != '/' && cp != bin_dir) {
    cp--;
  }
  if (cp == bin_dir) {
    strcpy(bin_dir, "./");
  } else {
    *++cp = '\0';
  }
  strcpy(lib_dir, bin_dir);
  strcat(lib_dir, "../lib/");
#endif /* UNIX */
}


int main(int argc, char *argv[])
{
  static char current_description[100];
  static char current_ctrl[100];
  static char current_data[100];
  int c;
  static char command[100];

  printf("\nD S P   S i m u l a t o r   V 1.0\n");
  set_directories(argv[0]);
  current_description[0] = '\0';
  setjmp(restart);
  while (1) {
    if (current_description[0] != '\0') {
      printf("\nYour currently selected description file is '%s'.\n",
	     current_description);
    }
    printf("\nFile, edit, simulate, help, or quit ");
    printf("(f, e, s, h, q)? ");
    fflush(stdout);
    gets(command);
    c = command[0];
    if (c == 'q' || c == 27) {
      break;
    }
    switch (c) {
      case 'f':
	printf("\nThere are the following description files present:\n");
	system("ls *.spl");
	printf("\nPlease enter one of these, or a name for a new file: ");
	gets(current_description);
	if (strchr(current_description, '.') == NULL) {
	  strcat(current_description, ".spl");
	}
	strcpy(current_ctrl, current_description);
	*strchr(current_ctrl, '.') = '\0';
	strcat(current_ctrl, ".ctl");
	strcpy(current_data, current_description);
	*strchr(current_data, '.') = '\0';
	strcat(current_data, ".dat");
	break;
      case 'e':
	if (current_description[0] == '\0') {
	  printf("\nPlease select a description file first!\n");
	  break;
	}
	sprintf(command, "%s %s", EDITOR, current_description);
	system(command);
	break;
      case 's':
	if (current_description[0] == '\0') {
	  printf("\nPlease select a description file first!\n");
	  break;
	}
	yyselect(current_description);
	ctrl_out = fopen(current_ctrl, "wt");
	if (ctrl_out == NULL) {
	  printf("\nCannot open simulation control output file!\n");
	  break;
	}
	data_out = fopen(current_data, "wb");
	if (data_out == NULL) {
	  printf("\nCannot open simulation data output file!\n");
	  break;
	}
	yyparse();
	fclose(ctrl_out);
	ctrl_out = NULL;
	fclose(data_out);
	data_out = NULL;
	view(current_ctrl, current_data);
	remove(current_ctrl);
	remove(current_data);
	break;
      case 'h':
      case '?':
	printf("\n");
	printf("file\t\tselect a DSP description file\n");
	printf("edit\t\tedit the selected DSP description file\n");
	printf("simulate\texecute the selected DSP description file\n");
	printf("help\t\tshow this help\n");
	printf("quit\t\texit simulator\n");
	break;
    }
  }
  printf("\nBye!\n\n");
  return 0;
}
