
%{

/*
 * DSPLAN.Y	DSP laguage grammar
 */


#include "control.h"
#include "memory.h"
#include "symtbl.h"
#include "tree.h"
#include "element.h"
#include "depend.h"
#include "simulate.h"

#include <stdio.h>

/* #define YYDEBUG		1 */

int in_macro_defn;

%}


%union {
  char		*sval;
  int		ival;
  double	fval;
  NODE		*nval;
}

%token		INCLUDE SIMULATE
%token		MACRO SIGNAL
%token	<sval>	IDENT
%token	<ival>	INUM
%token	<fval>	FNUM
%token	<sval>	STRING
%token		LPAR RPAR LCURL RCURL COMMA SEMI

%type	<nval>	top_description top_descriptor
%type	<sval>	head
%type	<nval>	body
%type	<nval>	description descriptor
%type	<nval>	call
%type	<nval>	arg_list args arg
%type	<ival>	simulation_cmd


%%


simulation	: top_description simulation_cmd
		  {
		    int number_signals;
		    ELEMENT *all_elements;
		    number_signals = get_number_global_signals();
#ifdef CHECK_NODES
		    printf("\n");
		    printf("Macro Definitions\n");
		    printf("-----------------\n");
		    show_macros();
		    printf("\n");
		    printf("DSP Description\n");
		    printf("---------------\n");
		    printf("%d signal(s)\n", number_signals);
		    show_nodelist($1, NULL, "\n", "\n");
#endif /* CHECK_NODES */
		    all_elements = gen_elements($1);
		    number_signals = get_number_global_signals();
#ifdef CHECK_ELEMENTS
		    printf("\n");
		    printf("DSP Elements\n");
		    printf("------------\n");
		    printf("%d signal(s)\n", number_signals);
		    show_elements(all_elements);
#endif /* CHECK_ELEMENTS */
		    all_elements = ordered_elements(all_elements,
						    number_signals);
#ifdef CHECK_ORDERING
		    printf("\n");
		    printf("DSP Elements in Simulation Order\n");
		    printf("--------------------------------\n");
		    printf("%d signal(s)\n", number_signals);
		    show_elements(all_elements);
#endif /* CHECK_ORDERING */
		    simulate(all_elements, number_signals, $2);
		    deallocate_arena(ARENA_1);
		    deallocate_arena(ARENA_2);
		  }
		;

top_description	: /* empty */
		  {
#ifdef YYDEBUG
		    yydebug = 1;
#endif /* YYDEBUG */
		    in_macro_defn = 0;
		    init_builtins();
		    init_macros();
		    init_global_signals();
		    $$ = NULL;
		  }
		| top_description top_descriptor
		  {
		    $$ = node_append($1, $2);
		  }
		;

top_descriptor	: file_incl
		  {
		    $$ = NULL;
		  }
		| macro_defn
		  {
		    $$ = NULL;
		  }
		| descriptor
		  {
		    $$ = $1;
		  }
		;

file_incl	: INCLUDE STRING
		  {
		    yyselect($2);
		  }
		;

macro_defn	: macro head body
		  {
		    NODE *macro;
		    macro = new_macro($2,
				      get_number_params(),
				      get_number_local_signals(),
				      $3);
		    enter_macro($2, macro);
		    in_macro_defn = 0;
		  }
		;

macro		: MACRO
		  {
		    in_macro_defn = 1;
		    init_params();
		    init_local_signals();
		  }
		;

head		: IDENT LPAR param_list RPAR
		  {
		    static char errmsg[ERRMSG_SIZE];
		    if (lookup_builtin($1) != NULL) {
		      sprintf(errmsg,
			      "file '%s', line %d, "
			      "re-definition of built-in '%s'",
			      yyfilename, yylineno, $1);
		      error(errmsg);
		    }
		    if (lookup_macro($1) != NULL) {
		      sprintf(errmsg,
			      "file '%s', line %d, "
			      "re-definition of macro '%s'",
			      yyfilename, yylineno, $1);
		      error(errmsg);
		    }
		    $$ = $1;
		  }
		;

body		: LCURL description RCURL
		  {
		    $$ = $2;
		  }
		;

param_list	: /* empty */
		| params
		;

params		: IDENT
		  {
		    enter_param($1);
		  }
		| params COMMA IDENT
		  {
		    enter_param($3);
		  }
		;

description	: /* empty */
		  {
		    $$ = NULL;
		  }
		| description descriptor
		  {
		    $$ = node_append($1, $2);
		  }
		;

descriptor	: signal_decl
		  {
		    $$ = NULL;
		  }
		| call
		  {
		    $$ = $1;
		  }
		;

signal_decl	: SIGNAL signals SEMI
		;

signals		: IDENT
		  {
		    static char errmsg[ERRMSG_SIZE];
		    if (in_macro_defn) {
		      if (lookup_param($1) != NULL) {
			sprintf(errmsg,
				"file '%s', line %d, "
				"'%s' already declared as parameter",
				yyfilename, yylineno, $1);
			error(errmsg);
		      }
		      enter_local_signal($1);
		    } else {
		      enter_global_signal($1);
		    }
		  }
		| signals COMMA IDENT
		  {
		    static char errmsg[ERRMSG_SIZE];
		    if (in_macro_defn) {
		      if (lookup_param($3) != NULL) {
			sprintf(errmsg,
				"file '%s', line %d, "
				"'%s' already declared as parameter",
				yyfilename, yylineno, $3);
			error(errmsg);
		      }
		      enter_local_signal($3);
		    } else {
		      enter_global_signal($3);
		    }
		  }
		;

call		: IDENT LPAR arg_list RPAR SEMI
		  {
		    int numargs;
		    SYMBOL *symbol;
		    static char errmsg[ERRMSG_SIZE];
		    numargs = nodelist_length($3);
		    if ((symbol = lookup_builtin($1)) != NULL) {
		      if (symbol->info.node->u.builtin.numparms != numargs) {
			sprintf(errmsg,
				"file '%s', line %d, "
				"wrong number args in call of builtin '%s'",
				yyfilename, yylineno, $1);
			error(errmsg);
		      }
		      $$ = new_call(symbol->info.node, $3);
		    } else
		    if ((symbol = lookup_macro($1)) != NULL) {
		      if (symbol->info.node->u.macro.numparms != numargs) {
			sprintf(errmsg,
				"file '%s', line %d, "
				"wrong number args in call of macro '%s'",
				yyfilename, yylineno, $1);
			error(errmsg);
		      }
		      $$ = new_call(symbol->info.node, $3);
		    } else {
		      sprintf(errmsg,
			      "file '%s', line %d, unknown symbol '%s'",
			      yyfilename, yylineno, $1);
		      error(errmsg);
		    }
		  }
		;

arg_list	: /* empty */
		  {
		    $$ = NULL;
		  }
		| args
		  {
		    $$ = $1;
		  }
		;

args		: arg
		  {
		    $$ = $1;
		  }
		| args COMMA arg
		  {
		    $$ = node_append($1, $3);
		  }
		;

arg		: IDENT
		  {
		    SYMBOL *symbol;
		    static char errmsg[ERRMSG_SIZE];
		    if (in_macro_defn &&
			(symbol = lookup_local_signal($1)) != NULL) {
		      $$ = new_arg_locsig(symbol->info.number);
		    } else
		    if (in_macro_defn &&
			(symbol = lookup_param($1)) != NULL) {
		      $$ = new_arg_param(symbol->info.number);
		    } else
		    if ((symbol = lookup_global_signal($1)) != NULL) {
		      $$ = new_arg_globsig(symbol->info.number);
		    } else {
		      sprintf(errmsg,
			      "file '%s', line %d, unknown symbol '%s'",
			      yyfilename, yylineno, $1);
		      error(errmsg);
		    }
		  }
		| INUM
		  {
		    $$ = new_arg_intval($1);
		  }
		| FNUM
		  {
		    $$ = new_arg_floval($1);
		  }
		;

simulation_cmd	: /* empty */
		  {
		    $$ = DEFAULT_STEPS;
		  }
		| SIMULATE INUM
		  {
		    $$ = $2;
		  }
		;


%%


void yyerror(char *msg)
{
  static char errmsg[ERRMSG_SIZE];

  sprintf(errmsg,
	  "file '%s', line %d, %s",
	  yyfilename, yylineno, msg);
  error(errmsg);
}
