
/*
 * MEMORY.C	memory management
 */


#include "memory.h"
#include "control.h"

#include <stdlib.h>


typedef struct chunk {
  struct chunk *next;
  /* here follows a properly sized array of bytes */
} CHUNK;


static CHUNK *arenas[NUM_ARENAS];


void *allocate(unsigned long size, int arena)
{
  CHUNK *p;

  p = (CHUNK *) malloc(sizeof(CHUNK) + size);
  if (p == NULL) {
    error("out of memory");
  }
  p->next = arenas[arena];
  arenas[arena] = p;
  return (void *) (p + 1);
}


void deallocate_arena(int arena)
{
  CHUNK *p;

  while (1) {
    p = arenas[arena];
    if (p == NULL) {
      break;
    }
    arenas[arena] = p->next;
    free(p);
  }
}


void deallocate_all(void)
{
  int i;

  for (i = 0; i < NUM_ARENAS; i++) {
    deallocate_arena(i);
  }
}
