
/*
 * TREE.C	parse tree routines
 */


#include "tree.h"
#include "memory.h"
#include "control.h"
#include "symtbl.h"

#include <stdio.h>
#include <stdlib.h>


NODE *new_builtin(char *name, int type, int numparms)
{
  NODE *node;

  node = allocate(sizeof(*node), ARENA_1);
  node->type = N_BUILTIN;
  node->u.builtin.name = name;
  node->u.builtin.type = type;
  node->u.builtin.numparms = numparms;
  node->next = NULL;
  return node;
}


NODE *new_macro(char *name, int numparms, int numlocals, NODE *calls)
{
  NODE *node;

  node = allocate(sizeof(*node), ARENA_1);
  node->type = N_MACRO;
  node->u.macro.name = name;
  node->u.macro.numparms = numparms;
  node->u.macro.numlocals = numlocals;
  node->u.macro.calls = calls;
  node->next = NULL;
  return node;
}


NODE *new_call(NODE *callee, NODE *args)
{
  NODE *node;

  node = allocate(sizeof(*node), ARENA_1);
  node->type = N_CALL;
  node->u.call.callee = callee;
  node->u.call.args = args;
  node->next = NULL;
  return node;
}


NODE *new_arg_globsig(int global_signal_number)
{
  NODE *node;

  node = allocate(sizeof(*node), ARENA_1);
  node->type = N_ARG_GLOBSIG;
  node->u.arg_globsig.signum = global_signal_number;
  node->next = NULL;
  return node;
}


NODE *new_arg_param(int param_number)
{
  NODE *node;

  node = allocate(sizeof(*node), ARENA_1);
  node->type = N_ARG_PARAM;
  node->u.arg_param.parmnum = param_number;
  node->next = NULL;
  return node;
}


NODE *new_arg_locsig(int local_signal_number)
{
  NODE *node;

  node = allocate(sizeof(*node), ARENA_1);
  node->type = N_ARG_LOCSIG;
  node->u.arg_locsig.signum = local_signal_number;
  node->next = NULL;
  return node;
}


NODE *new_arg_intval(int value)
{
  NODE *node;

  node = allocate(sizeof(*node), ARENA_1);
  node->type = N_ARG_INTVAL;
  node->u.arg_intval.value = value;
  node->next = NULL;
  return node;
}


NODE *new_arg_floval(double value)
{
  NODE *node;

  node = allocate(sizeof(*node), ARENA_1);
  node->type = N_ARG_FLOVAL;
  node->u.arg_floval.value = value;
  node->next = NULL;
  return node;
}


NODE *node_append(NODE *list, NODE *node)
{
  NODE *aux;

  if (list == NULL) {
    return node;
  }
  if (node == NULL) {
    return list;
  }
  node->next = NULL;
  aux = list;
  while (aux->next != NULL) {
    aux = aux->next;
  }
  aux->next = node;
  return list;
}


int nodelist_length(NODE *list)
{
  int length;

  length = 0;
  while (list != NULL) {
    length++;
    list = list->next;
  }
  return length;
}


void show_node(NODE *node, int expand_macros)
{
  static char errmsg[ERRMSG_SIZE];

  switch (node->type) {
    case N_BUILTIN:
	printf("BUILTIN %s", node->u.builtin.name);
	break;
    case N_MACRO:
	printf("MACRO %s", node->u.macro.name);
	if (expand_macros) {
	  printf(", %d parameter(s), %d local signal(s)",
	       node->u.macro.numparms, node->u.macro.numlocals);
	  show_nodelist(node->u.macro.calls, "\n\t", "\n\t", "\n");
	}
	break;
    case N_CALL:
	printf("CALL ");
	show_node(node->u.call.callee, 0);
	show_nodelist(node->u.call.args, " ( ", ", ", " )");
	break;
    case N_ARG_GLOBSIG:
	printf("GLB%d", node->u.arg_globsig.signum);
	break;
    case N_ARG_PARAM:
	printf("PAR%d", node->u.arg_param.parmnum);
	break;
    case N_ARG_LOCSIG:
	printf("LOC%d", node->u.arg_locsig.signum);
	break;
    case N_ARG_INTVAL:
	printf("%d", node->u.arg_intval.value);
	break;
    case N_ARG_FLOVAL:
	printf("%e", node->u.arg_floval.value);
	break;
    default:
	sprintf(errmsg,
		"cannot show unknown node type %d",
		node->type);
	error(errmsg);
	break;
  }
}


void show_nodelist(NODE *list, char *leader, char *separator, char *trailer)
{
  if (leader != NULL) {
    printf("%s", leader);
  }
  while (list != NULL) {
    show_node(list, 0);
    if (list->next != NULL) {
      if (separator != NULL) {
	printf("%s", separator);
      }
    }
    list = list->next;
  }
  if (trailer != NULL) {
    printf("%s", trailer);
  }
}
