
/*
 * TREE.H	parse tree routines
 */


#ifndef _TREE_H_
#define _TREE_H_


#define N_BUILTIN	0
#define N_MACRO		1
#define N_CALL		2
#define N_ARG_GLOBSIG	3
#define N_ARG_PARAM	4
#define N_ARG_LOCSIG	5
#define N_ARG_INTVAL	6
#define N_ARG_FLOVAL	7

#define B_CONNECT	0
#define B_ADD		1
#define B_SUB		2
#define B_MUL		3
#define B_DIV		4
#define B_DELAY		5
#define B_PROBE		6
#define B_STEP		7
#define B_SIN		8
#define B_COS		9
#define B_ATAN		10
#define B_SQRT		11

typedef struct node {
  int type;
  union {
    struct {
      char *name;
      int type;
      int numparms;
    } builtin;
    struct {
      char *name;
      int numparms;
      int numlocals;
      struct node *calls;
    } macro;
    struct {
      struct node *callee;
      struct node *args;
    } call;
    struct {
      int signum;
    } arg_globsig;
    struct {
      int parmnum;
    } arg_param;
    struct {
      int signum;
    } arg_locsig;
    struct {
      int value;
    } arg_intval;
    struct {
      double value;
    } arg_floval;
  } u;
  struct node *next;
} NODE;


NODE *new_builtin(char *name, int type, int numparms);
NODE *new_macro(char *name, int numparms, int numlocals, NODE *calls);
NODE *new_call(NODE *callee, NODE *args);
NODE *new_arg_globsig(int global_signal_number);
NODE *new_arg_param(int param_number);
NODE *new_arg_locsig(int local_signal_number);
NODE *new_arg_intval(int value);
NODE *new_arg_floval(double value);
NODE *node_append(NODE *list, NODE *node);
int nodelist_length(NODE *list);
void show_node(NODE *node, int expand_macros);
void show_nodelist(NODE *list, char *leader, char *separator, char *trailer);


#endif /* _TREE_H_ */
