
/*
 * Oscilloscope screen widget
 */


#include "tk.h"

#include <stdlib.h>
#include <math.h>


/* widget record */

typedef struct {
  Tk_Window tkwin;
  Display *display;
  Tcl_Interp *interp;
  int xunits;			/* number of units in x direction */
  int ppxunit;			/* pixels per unit in x direction */
  int yunits;			/* number of units in y direction */
  int ppyunit;			/* pixels per unit in y direction */
  Tk_3DBorder backColor;	/* background color */
  Tk_3DBorder gridColor;	/* grid color */
  Tk_3DBorder signalColor;	/* signal color */
  int xPos;
  double xCoeff;
  int interpolation[4];
  int lowpassfilter[4];
  int yPos[4];
  double yCoeff[4];
  int signal[4];		/* channel-to-signal mapping */
  FILE *dataFile;
  int numberCycles;
  int numberSignals;
  GC gc;			/* graphics context */
  int updatePending;		/* is update pending? */
} ScreenData;


/* configuration table */

static Tk_ConfigSpec configSpecs[] = {
  { TK_CONFIG_PIXELS,
    "-xunits",
    "xunits",
    "Xunits",
    "14",
    Tk_Offset(ScreenData, xunits),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_PIXELS,
    "-ppxunit",
    "ppxunit",
    "PPXunit",
    "30",
    Tk_Offset(ScreenData, ppxunit),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_PIXELS,
    "-yunits",
    "yunits",
    "Yunits",
    "8",
    Tk_Offset(ScreenData, yunits),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_PIXELS,
    "-ppyunit",
    "ppyunit",
    "PPYunit",
    "30",
    Tk_Offset(ScreenData, ppyunit),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_BORDER,
    "-background",
    "background",
    "Background",
    "#006400",
    Tk_Offset(ScreenData, backColor),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_SYNONYM,
    "-bg",
    "background",
    (char *) NULL,
    (char *) NULL,
    0,
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_BORDER,
    "-gridcolor",
    "gridcolor",
    "Gridcolor",
    "#000000",
    Tk_Offset(ScreenData, gridColor),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_SYNONYM,
    "-gc",
    "gridcolor",
    (char *) NULL,
    (char *) NULL,
    0,
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_BORDER,
    "-signalcolor",
    "signalcolor",
    "Signalcolor",
    "#FFFFFF",
    Tk_Offset(ScreenData, signalColor),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_SYNONYM,
    "-sc",
    "signalcolor",
    (char *) NULL,
    (char *) NULL,
    0,
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_INT,
    "-xpos",
    "xpos",
    "Xpos",
    "0",
    Tk_Offset(ScreenData, xPos),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_DOUBLE,
    "-xcoeff",
    "xcoeff",
    "Xcoeff",
    "10.0",
    Tk_Offset(ScreenData, xCoeff),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_BOOLEAN,
    "-interpolation0",
    "interpolation0",
    "Interpolation0",
    "false",
    Tk_Offset(ScreenData, interpolation[0]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_BOOLEAN,
    "-interpolation1",
    "interpolation1",
    "Interpolation1",
    "false",
    Tk_Offset(ScreenData, interpolation[1]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_BOOLEAN,
    "-interpolation2",
    "interpolation2",
    "Interpolation2",
    "false",
    Tk_Offset(ScreenData, interpolation[2]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_BOOLEAN,
    "-interpolation3",
    "interpolation3",
    "Interpolation3",
    "false",
    Tk_Offset(ScreenData, interpolation[3]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_BOOLEAN,
    "-lowpassfilter0",
    "lowpassfilter0",
    "Lowpassfilter0",
    "false",
    Tk_Offset(ScreenData, lowpassfilter[0]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_BOOLEAN,
    "-lowpassfilter1",
    "lowpassfilter1",
    "Lowpassfilter1",
    "false",
    Tk_Offset(ScreenData, lowpassfilter[1]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_BOOLEAN,
    "-lowpassfilter2",
    "lowpassfilter2",
    "Lowpassfilter2",
    "false",
    Tk_Offset(ScreenData, lowpassfilter[2]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_BOOLEAN,
    "-lowpassfilter3",
    "lowpassfilter3",
    "Lowpassfilter3",
    "false",
    Tk_Offset(ScreenData, lowpassfilter[3]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_INT,
    "-ypos0",
    "ypos0",
    "Ypos0",
    "-90",
    Tk_Offset(ScreenData, yPos[0]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_INT,
    "-ypos1",
    "ypos1",
    "Ypos1",
    "-30",
    Tk_Offset(ScreenData, yPos[1]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_INT,
    "-ypos2",
    "ypos2",
    "Ypos2",
    "30",
    Tk_Offset(ScreenData, yPos[2]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_INT,
    "-ypos3",
    "ypos3",
    "Ypos3",
    "90",
    Tk_Offset(ScreenData, yPos[3]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_DOUBLE,
    "-ycoeff0",
    "ycoeff0",
    "Ycoeff0",
    "1.0",
    Tk_Offset(ScreenData, yCoeff[0]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_DOUBLE,
    "-ycoeff1",
    "ycoeff1",
    "Ycoeff1",
    "1.0",
    Tk_Offset(ScreenData, yCoeff[1]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_DOUBLE,
    "-ycoeff2",
    "ycoeff2",
    "Ycoeff2",
    "1.0",
    Tk_Offset(ScreenData, yCoeff[2]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_DOUBLE,
    "-ycoeff3",
    "ycoeff3",
    "Ycoeff3",
    "1.0",
    Tk_Offset(ScreenData, yCoeff[3]),
    0,
    (Tk_CustomOption *) NULL },
  { TK_CONFIG_END,
    (char *) NULL,
    (char *) NULL,
    (char *) NULL,
    (char *) NULL,
    0,
    0,
    (Tk_CustomOption *) NULL }
};


/* prototypes */

static int ScreenCmd(ClientData clientData,
                     Tcl_Interp *interp,
                     int argc, char *argv[]);
static int ScreenWidgetCmd(ClientData clientData,
                           Tcl_Interp *interp,
                           int argc, char *argv[]);
static int ScreenConfigure(Tcl_Interp *interp,
                           ScreenData *screenPtr,
                           int argc, char *argv[],
                           int flags);
static void ScreenEventProc(ClientData clientData,
                            XEvent *eventPtr);
static void ScreenDisplay(ClientData clientData);
static void ScreenDestroy(char *blockPtr);
int Screen_Init(Tcl_Interp *interp);


/* class command procedure */

static int ScreenCmd(ClientData clientData,
                     Tcl_Interp *interp,
                     int argc, char *argv[])
{
  Tk_Window main;
  Tk_Window tkwin;
  ScreenData *screenPtr;
  int channel;

  if (argc < 9) {
    Tcl_AppendResult(interp,
                     "wrong # args: should be \"",
                     argv[0],
                     " pathName signal0 signal1 signal2 signal3"
                     " fileName cycles signals ?options?\"",
                     (char *) NULL);
    return TCL_ERROR;
  }

  main = (Tk_Window) clientData;
  tkwin = Tk_CreateWindowFromPath(interp, main,
                                  argv[1], (char *) NULL);
  if (tkwin == NULL) {
    return TCL_ERROR;
  }
  Tk_SetClass(tkwin, "Screen");

  screenPtr = (ScreenData *) malloc(sizeof(ScreenData));
  screenPtr->tkwin = tkwin;
  screenPtr->display = Tk_Display(tkwin);
  screenPtr->interp = interp;
  screenPtr->xunits = 14;
  screenPtr->ppxunit = 30;
  screenPtr->yunits = 8;
  screenPtr->ppyunit = 30;
  screenPtr->backColor = NULL;
  screenPtr->gridColor = NULL;
  screenPtr->signalColor = NULL;
  screenPtr->xPos = 0;
  screenPtr->xCoeff = 10.0;
  for (channel = 0; channel < 4; channel++) {
    screenPtr->interpolation[channel] = 0;
    screenPtr->lowpassfilter[channel] = 0;
    screenPtr->yPos[channel] = (channel * 2 - 3) * screenPtr->ppyunit;
    screenPtr->yCoeff[channel] = 1.0;
    screenPtr->signal[channel] = atoi(argv[2 + channel]);
  }
  screenPtr->dataFile = fopen(argv[6], "rb");
  if (screenPtr->dataFile == NULL) {
    Tcl_AppendResult(interp,
                     "cannot open data file \"", argv[6], "\"", NULL);
    return TCL_ERROR;
  }
  screenPtr->numberCycles = atoi(argv[7]);
  if (screenPtr->numberCycles <= 0) {
    Tcl_AppendResult(interp,
                     "illegal number of cycles: \"", argv[7], "\"", NULL);
    return TCL_ERROR;
  }
  screenPtr->numberSignals = atoi(argv[8]);
  if (screenPtr->numberSignals <= 0) {
    Tcl_AppendResult(interp,
                     "illegal number of signals: \"", argv[8], "\"", NULL);
    return TCL_ERROR;
  }
  screenPtr->gc = None;
  screenPtr->updatePending = 0;

  Tk_CreateEventHandler(screenPtr->tkwin,
                        ExposureMask | StructureNotifyMask,
                        ScreenEventProc,
                        (ClientData) screenPtr);
  Tcl_CreateCommand(interp,
                    Tk_PathName(screenPtr->tkwin),
                    ScreenWidgetCmd,
                    (ClientData) screenPtr,
                    (Tcl_CmdDeleteProc *) NULL);
  if (ScreenConfigure(interp, screenPtr,
                      argc - 9, argv + 9, 0) != TCL_OK) {
    Tk_DestroyWindow(screenPtr->tkwin);
    return TCL_ERROR;
  }
  interp->result = Tk_PathName(screenPtr->tkwin);
  return TCL_OK;
}


/* widget command procedure */

static int ScreenWidgetCmd(ClientData clientData,
                           Tcl_Interp *interp,
                           int argc, char *argv[])
{
  ScreenData *screenPtr;
  int result;

  screenPtr = (ScreenData *) clientData;
  result = TCL_OK;
  if (argc < 2) {
    Tcl_AppendResult(interp,
                     "wrong # args: should be \"",
                     argv[0],
                     " option ?arg arg ...?\"",
                     (char *) NULL);
    return TCL_ERROR;
  }
  Tk_Preserve((ClientData) screenPtr);
  if (strcmp(argv[1], "configure") == 0) {
    if (argc == 2) {
      result = Tk_ConfigureInfo(interp,
                                screenPtr->tkwin,
                                configSpecs,
                                (char *) screenPtr,
                                (char *) NULL,
                                0);
    } else
    if (argc == 3) {
      result = Tk_ConfigureInfo(interp,
                                screenPtr->tkwin,
                                configSpecs,
                                (char *) screenPtr,
                                argv[2],
                                0);
    } else {
      result = ScreenConfigure(interp,
                               screenPtr,
                               argc - 2, argv + 2,
                               TK_CONFIG_ARGV_ONLY);
    }
  } else {
    Tcl_AppendResult(interp,
                     "bad option \"",
                     argv[1],
                     "\": must be configure",
                     (char *) NULL);
    goto error;
  }
  if (!screenPtr->updatePending) {
    Tk_DoWhenIdle(ScreenDisplay, (ClientData) screenPtr);
    screenPtr->updatePending = 1;
  }
  Tk_Release((ClientData) screenPtr);
  return result;
  
error:
  Tk_Release((ClientData) screenPtr);
  return TCL_ERROR;
}


/* configuration procedure */

static int ScreenConfigure(Tcl_Interp *interp,
                           ScreenData *screenPtr,
                           int argc, char *argv[],
                           int flags)
{
  if (Tk_ConfigureWidget(interp, screenPtr->tkwin,
                         configSpecs, argc, argv,
                         (char *) screenPtr, flags) != TCL_OK) {
    return TCL_ERROR;
  }
  Tk_SetWindowBackground(screenPtr->tkwin,
                         Tk_3DBorderColor(screenPtr->backColor)->pixel);
  if (screenPtr->gc == None) {
    XGCValues gcValues;
    gcValues.function = GXcopy;
    gcValues.graphics_exposures = False;
    screenPtr->gc = Tk_GetGC(screenPtr->tkwin,
                             GCFunction | GCGraphicsExposures,
                             &gcValues);
  }
  Tk_GeometryRequest(screenPtr->tkwin,
                     screenPtr->xunits * screenPtr->ppxunit,
                     screenPtr->yunits * screenPtr->ppyunit);
  Tk_SetInternalBorder(screenPtr->tkwin, 0);
  if (!screenPtr->updatePending) {
    Tk_DoWhenIdle(ScreenDisplay, (ClientData) screenPtr);
    screenPtr->updatePending = 1;
  }
  return TCL_OK;
}


/* event handler */

static void ScreenEventProc(ClientData clientData,
                            XEvent *eventPtr)
{
  ScreenData *screenPtr;

  screenPtr = (ScreenData *) clientData;
  if (eventPtr->type == Expose) {
    if (!screenPtr->updatePending) {
      Tk_DoWhenIdle(ScreenDisplay, (ClientData) screenPtr);
      screenPtr->updatePending = 1;
    }
  } else
  if (eventPtr->type == ConfigureNotify) {
    if (!screenPtr->updatePending) {
      Tk_DoWhenIdle(ScreenDisplay, (ClientData) screenPtr);
      screenPtr->updatePending = 1;
    }
  } else
  if (eventPtr->type == DestroyNotify) {
    Tcl_DeleteCommand(screenPtr->interp,
                      Tk_PathName(screenPtr->tkwin));
    screenPtr->tkwin = NULL;
    if (screenPtr->updatePending) {
      Tk_CancelIdleCall(ScreenDisplay, (ClientData) screenPtr);
    }
    Tk_EventuallyFree((ClientData) screenPtr, ScreenDestroy);
  }
}


/* display procedure */

static void drawGrid(Display *display,
                     Pixmap pm,
                     GC gc,
                     ScreenData *screenPtr,
                     int w,
                     int h)
{
  int i;

  XSetForeground(display,
                 gc,
                 Tk_3DBorderColor(screenPtr->gridColor)->pixel);
  for (i = 1; i < screenPtr->yunits; i++) {
    XDrawLine(display, pm, gc,
              0, i * screenPtr->ppyunit,
              w, i * screenPtr->ppyunit);
  }
  for (i = 1; i < 5 * screenPtr->yunits; i++) {
    XDrawLine(display, pm, gc,
              w / 2 - screenPtr->ppxunit / 10, i * screenPtr->ppyunit / 5,
              w / 2 + screenPtr->ppxunit / 10, i * screenPtr->ppyunit / 5);
  }
  for (i = 1; i < screenPtr->xunits; i++) {
    XDrawLine(display, pm, gc,
              i * screenPtr->ppxunit, 0,
              i * screenPtr->ppxunit, h);
  }
  for (i = 1; i < 5 * screenPtr->xunits; i++) {
    XDrawLine(display, pm, gc,
              i * screenPtr->ppxunit / 5, h / 2 - screenPtr->ppyunit / 10,
              i * screenPtr->ppxunit / 5, h / 2 + screenPtr->ppyunit / 10);
  }
}


static double getDataValue(FILE *dataFile,
                           int cycle,
                           int numberCycles,
                           int signal,
                           int numberSignals)
{
  double value;

  if (dataFile == NULL ||
      cycle < 0 || cycle >= numberCycles ||
      signal < 0 || signal >= numberSignals) {
    return 0.0;
  }
  fseek(dataFile,
        ((long) cycle * numberSignals + signal) * sizeof(double),
        SEEK_SET);
  fread(&value, sizeof(double), 1, dataFile);
  return value;
}


static void drawChannelConst(Display *display,
                             Pixmap pm,
                             GC gc,
                             ScreenData *screenPtr,
                             int channel,
                             int w,
                             int h)
{
  FILE *dataFile;
  int numberCycles;
  int numberSignals;
  int signal;
  double xScale;
  int index0, index;
  double yScale;
  int yOffset;
  int x1, y1, x2, y2;
  int dx;

  XSetForeground(display,
                 gc,
                 Tk_3DBorderColor(screenPtr->signalColor)->pixel);
  dataFile = screenPtr->dataFile;
  numberCycles = screenPtr->numberCycles;
  numberSignals = screenPtr->numberSignals;
  signal = screenPtr->signal[channel];
  xScale = screenPtr->ppxunit / screenPtr->xCoeff;
  index0 = screenPtr->xPos;
  index = index0;
  yScale = - screenPtr->ppyunit / screenPtr->yCoeff[channel];
  yOffset = h / 2 + screenPtr->yPos[channel];
  x1 = 0;
  y1 = getDataValue(dataFile, index, numberCycles,
                    signal, numberSignals) * yScale + yOffset;
  while (1) {
    index++;
    x2 = (index - index0) * xScale;
    if (x2 >= w) {
      break;
    }
    y2 = getDataValue(dataFile, index, numberCycles,
                      signal, numberSignals) * yScale + yOffset;
    dx = x2 - x1 - 1;
    if (dx < 0) {
      dx = 0;
    }
    XDrawLine(display, pm, gc, x1, y1, x1 + dx, y1);
    x1 = x2;
    y1 = y2;
  }
}


static void drawChannelLinear(Display *display,
                              Pixmap pm,
                              GC gc,
                              ScreenData *screenPtr,
                              int channel,
                              int w,
                              int h)
{
  FILE *dataFile;
  int numberCycles;
  int numberSignals;
  int signal;
  double xScale;
  int index0, index;
  double yScale;
  int yOffset;
  int x1, y1, x2, y2;

  XSetForeground(display,
                 gc,
                 Tk_3DBorderColor(screenPtr->signalColor)->pixel);
  dataFile = screenPtr->dataFile;
  numberCycles = screenPtr->numberCycles;
  numberSignals = screenPtr->numberSignals;
  signal = screenPtr->signal[channel];
  xScale = screenPtr->ppxunit / screenPtr->xCoeff;
  index0 = screenPtr->xPos;
  index = index0;
  yScale = - screenPtr->ppyunit / screenPtr->yCoeff[channel];
  yOffset = h / 2 + screenPtr->yPos[channel];
  x1 = 0;
  y1 = getDataValue(dataFile, index, numberCycles,
                    signal, numberSignals) * yScale + yOffset;
  while (1) {
    index++;
    x2 = (index - index0) * xScale;
    if (x2 >= w) {
      break;
    }
    y2 = getDataValue(dataFile, index, numberCycles,
                      signal, numberSignals) * yScale + yOffset;
    XDrawLine(display, pm, gc, x1, y1, x2, y2);
    x1 = x2;
    y1 = y2;
  }
}


#define N	8	/* 2*N sample values are used during interpolation */
#define M	8	/* M points are interpolated per sample */
#define EPS	1e-10	/* numerically considered as zero */


static double interpolate(double *u, double alpha)
{
  double result;
  int k;

  result = 0.0;
  for (k = -N + 1; k <= N; k++) {
    if (fabs(alpha - k) < EPS) {
      result += u[k + N - 1];
    } else {
      result += u[k + N - 1] * sin(PI * (alpha - k)) / (PI * (alpha - k));
    }
  }
  return result;
}


static void drawChannelFilter(Display *display,
                              Pixmap pm,
                              GC gc,
                              ScreenData *screenPtr,
                              int channel,
                              int w,
                              int h)
{
  FILE *dataFile;
  int numberCycles;
  int numberSignals;
  int signal;
  double xScale;
  int index0, index;
  double yScale;
  int yOffset;
  double y[2 * N];
  int k;
  double alpha;
  int x1, y1, x2, y2;
  int i;

  XSetForeground(display,
                 gc,
                 Tk_3DBorderColor(screenPtr->signalColor)->pixel);
  dataFile = screenPtr->dataFile;
  numberCycles = screenPtr->numberCycles;
  numberSignals = screenPtr->numberSignals;
  signal = screenPtr->signal[channel];
  xScale = screenPtr->ppxunit / screenPtr->xCoeff;
  index0 = screenPtr->xPos;
  index = index0;
  yScale = - screenPtr->ppyunit / screenPtr->yCoeff[channel];
  yOffset = h / 2 + screenPtr->yPos[channel];
  /* preset data array */
  for (k = -N + 1; k <= N; k++) {
    y[k + N - 1] = getDataValue(dataFile, index + k, numberCycles,
                                signal, numberSignals);
  }
  /* loop on sample points */
  while (1) {
    /* interpolate */
    alpha = 0;
    x1 = (index - index0 + alpha) * xScale;
    y1 = interpolate(y, alpha) * yScale + yOffset;
    for (i = 0; i < M; i++) {
      alpha += 1.0 / M;
      x2 = (index - index0 + alpha) * xScale;
      if (x2 >= w) {
        break;
      }
      y2 = interpolate(y, alpha) * yScale + yOffset;
      XDrawLine(display, pm, gc, x1, y1, x2, y2);
      x1 = x2;
      y1 = y2;
    }
    /* shift data array */
    for (k = 0; k < 2 * N - 1; k++) {
      y[k] = y[k + 1];
    }
    /* get new sample point */
    index++;
    if ((index - index0) * xScale >= w) {
      break;
    }
    y[2 * N - 1] = getDataValue(dataFile, index + N, numberCycles,
                                signal, numberSignals);
  }
}


static void ScreenDisplay(ClientData clientData)
{
  ScreenData *screenPtr;
  Tk_Window tkwin;
  Display *display;
  GC gc;
  int w;
  int h;
  Pixmap pm;
  int channel;

  screenPtr = (ScreenData *) clientData;
  tkwin = screenPtr->tkwin;
  display = Tk_Display(tkwin);
  gc = screenPtr->gc;
  w = Tk_Width(tkwin);
  h = Tk_Height(tkwin);

  screenPtr->updatePending = 0;
  if (!Tk_IsMapped(tkwin)) {
    return;
  }
  
  pm = XCreatePixmap(display,
                     Tk_WindowId(tkwin),
                     w, h,
                     Tk_Depth(tkwin));

  /* background */
  Tk_Fill3DRectangle(tkwin, pm,
                     screenPtr->backColor,
                     0, 0,
                     w, h,
                     0, TK_RELIEF_FLAT);
  /* grid */
  drawGrid(display, pm, gc, screenPtr, w, h);
  /* signals */
  for (channel = 0; channel < 4; channel++) {
    if (!screenPtr->interpolation[channel]) {
      drawChannelConst(display, pm, gc, screenPtr, channel, w, h);
    } else {
      if (!screenPtr->lowpassfilter[channel]) {
        drawChannelLinear(display, pm, gc, screenPtr, channel, w, h);
      } else {
        drawChannelFilter(display, pm, gc, screenPtr, channel, w, h);
      }
    }
  }

  XCopyArea(display, pm,
            Tk_WindowId(tkwin),
            gc,
            0, 0,
            w, h,
            0, 0);
  XFreePixmap(display, pm);
}


/* destroy procedure */

static void ScreenDestroy(char *blockPtr)
{
  ScreenData *screenPtr;
  
  screenPtr = (ScreenData *) blockPtr;
  Tk_FreeOptions(configSpecs,
                 (char *) screenPtr,
                 screenPtr->display,
                 0);
  if (screenPtr->gc != None) {
    Tk_FreeGC(screenPtr->display, screenPtr->gc);
  }
  fclose(screenPtr->dataFile);
  free(screenPtr);
}


/* package initialization */

int Screen_Init(Tcl_Interp *interp)
{
  Tcl_CreateCommand(interp,
                    "screen",
                    ScreenCmd,
                    (ClientData) Tk_MainWindow(interp),
                    (Tcl_CmdDeleteProc *) NULL);
  return TCL_OK;
}
