
/*
 * Interface to DSP simulator
 */


#include "control.h"
#include "memory.h"

#include <tcl.h>
#include <stdio.h>
#include <string.h>
#include <setjmp.h>


char infileName[200];
char lib_dir[200];
char ctrlfileName[200];
char datafileName[200];

FILE *ctrl_out = NULL;
FILE *data_out = NULL;
static char error_message[500];
static jmp_buf restart;


void error(char *msg)
{
  strcpy(error_message, msg);
  yycleanup();
  if (ctrl_out != NULL) {
    fclose(ctrl_out);
    ctrl_out = NULL;
  }
  if (data_out != NULL) {
    fclose(data_out);
    data_out = NULL;
  }
  deallocate_all();
  longjmp(restart, 1);
}


int Simulator_Run(ClientData clientData,
                  Tcl_Interp *interp,
                  int objc, Tcl_Obj *CONST objv[])
{
  char *text1;
  int length1;
  char *text2;
  int length2;
  char *text3;
  int length3;
  char *text4;
  int length4;
  Tcl_Obj *resPtr;

  if (objc != 5) {
    Tcl_WrongNumArgs(interp, 1, objv,
                     "inputFile libraryDir ctrlOutputFile dataOutputFile");
    return TCL_ERROR;
  }
  text1 = Tcl_GetStringFromObj(objv[1], &length1);
  strncpy(infileName, text1, length1);
  infileName[length1] = '\0';
  text2 = Tcl_GetStringFromObj(objv[2], &length2);
  strncpy(lib_dir, text2, length2);
  lib_dir[length2] = '/';
  lib_dir[length2 + 1] = '\0';
  text3 = Tcl_GetStringFromObj(objv[3], &length3);
  strncpy(ctrlfileName, text3, length3);
  ctrlfileName[length3] = '\0';
  text4 = Tcl_GetStringFromObj(objv[4], &length4);
  strncpy(datafileName, text4, length4);
  datafileName[length4] = '\0';
  if (setjmp(restart) == 0) {
    /* first call: no error */
    yyselect(infileName);
    ctrl_out = fopen(ctrlfileName, "wt");
    data_out = fopen(datafileName, "wb");
    yyparse();
    fclose(ctrl_out);
    ctrl_out = NULL;
    fclose(data_out);
    data_out = NULL;
    return TCL_OK;
  } else {
    /* second call: error */
    resPtr = Tcl_GetObjResult(interp);
    Tcl_SetStringObj(resPtr, error_message, -1);
    return TCL_OK;
  }
}


/*------------------------------------------------------------*/


int Simulator_Init(Tcl_Interp *interp)
{
  Tcl_CreateObjCommand(interp, "Simulator_Run", Simulator_Run,
                       NULL, NULL);
  return TCL_OK;
}
