
/*
 * VIEW.C	view DSP simulation results
 */


#include "view.h"
#include "control.h"

#include <stdio.h>


#define MAX_NUM_SIGNALS		10
#define MAX_NAME_LENGTH		100


static FILE *ctrl_file;
static FILE *data_file;
static int number_cycles;
static int number_signals;
static char signal_name[MAX_NUM_SIGNALS][MAX_NAME_LENGTH];


double get_data_value(int cycle, int channel)
{
  double value;
  
  if (cycle < 0 || cycle >= number_cycles) {
    return 0.0;
  }
  fseek(data_file,
        ((long) cycle * number_signals + channel) * sizeof(double),
        SEEK_SET);
  fread(&value, sizeof(double), 1, data_file);
  return value;
}


void view(char *ctrl_filename, char *data_filename)
{
  static char buffer[100];
  static char errmsg[ERRMSG_SIZE];
  int i, j;
    
  ctrl_file = fopen(ctrl_filename, "rt");
  if (ctrl_file == NULL) {
    sprintf(errmsg,
            "cannot open simulation control result file '%s'",
            ctrl_filename);
    error(errmsg);
  }
  data_file = fopen(data_filename, "rb");
  if (data_file == NULL) {
    sprintf(errmsg,
            "cannot open simulation data result file '%s'",
            data_filename);
    error(errmsg);
  }
  fgets(buffer, 100, ctrl_file);
  sscanf(buffer, "%d", &number_cycles);
  fgets(buffer, 100, ctrl_file);
  sscanf(buffer, "%d", &number_signals);
  if (number_signals > MAX_NUM_SIGNALS) {
    error("too many signals in result file");
  }
  for (i = 0; i < number_signals; i++) {
    fgets(signal_name[i], MAX_NAME_LENGTH, ctrl_file);
    for (j = 0; j < MAX_NAME_LENGTH; j++) {
      if (signal_name[i][j] == '\n') {
        signal_name[i][j] = '\0';
        break;
      }
    }
  }
  printf("\n");
  printf("number of cycles  : %d\n", number_cycles);
  printf("number of signals : %d\n", number_signals);
  for (i = 0; i < number_signals; i++) {
    printf("name of signal[%d] : '%s'\n", i, signal_name[i]);
  }
  for (i = 0; i < number_cycles; i++) {
    printf("cycle %5d       :", i);
    for (j = 0; j < number_signals; j++) {
      printf("%10.2e ", get_data_value(i, j));
    }
    printf("\n");
  }
  fclose(ctrl_file);
  fclose(data_file);
}
