
#--------------------------------------------------------------#
# Help System based on HTML files
#--------------------------------------------------------------#

#--------------------------------------------------------------#
# auxiliary functions
#

set HELP_ToplevelWindowCount 0

proc HELP_newHelpWindow {} {
  global HELP_ToplevelWindowCount
  incr HELP_ToplevelWindowCount
  set toplevelWindow [toplevel .help$HELP_ToplevelWindowCount]
  wm title $toplevelWindow "Help $HELP_ToplevelWindowCount"
  menu $toplevelWindow.menubar
  $toplevelWindow config -menu $toplevelWindow.menubar
  foreach m {File} {
    set $m [menu $toplevelWindow.menubar.m$m -tearoff no]
    $toplevelWindow.menubar add cascade -label $m -underline 0 \
      -menu $toplevelWindow.menubar.m$m
  }
  set helpWindow [text $toplevelWindow.text -width 64 -height 24 \
    -yscrollcommand [list $toplevelWindow.yscrollbar set] \
    -padx 3 -pady 3 -takefocus no \
  ]
  set ysbarWindow [scrollbar $toplevelWindow.yscrollbar -orient vertical \
    -command [list $helpWindow yview] \
  ]
  $File add command -label "Index" -underline 0 \
    -command [list HELP_doIndex $helpWindow]
  $File add separator
  $File add command -label "Back" -underline 0 \
    -command [list HELP_doBack $helpWindow]
  $File add separator
  $File add command -label "Close" -underline 0 \
    -command [list HELP_doClose $helpWindow]
  pack $ysbarWindow -side right -fill y -expand no
  pack $helpWindow -side right -fill both -expand yes
  wm withdraw $toplevelWindow
  update idletasks
  wm deiconify $toplevelWindow
  return $helpWindow
}

proc HELP_doIndex {helpWindow} {
  global HELP_Info
  HELP_render $helpWindow $HELP_Info($helpWindow-Index)
}

proc HELP_doBack {helpWindow} {
  global HELP_Info
  if {[llength $HELP_Info($helpWindow-Stack)] > 1} {
    set page [lindex $HELP_Info($helpWindow-Stack) 1]
    set HELP_Info($helpWindow-Stack) \
        [lrange $HELP_Info($helpWindow-Stack) 2 end]
    HELP_render $helpWindow $page
  }
}

proc HELP_doClose {helpWindow} {
  global HELP_Info
  HMreset_win $helpWindow
  unset HELP_Info($helpWindow-Index)
  unset HELP_Info($helpWindow-Current)
  unset HELP_Info($helpWindow-Stack)
  destroy [winfo parent $helpWindow]
}

proc HELP_render {helpWindow file} {
  global HELP_Info
  set HELP_Info($helpWindow-Stack) \
      [linsert $HELP_Info($helpWindow-Stack) 0 $file]
  set fragment ""
  regexp {([^#]*)#(.+)} $file dummy file fragment
  if {$file == "" && $fragment != ""} {
    HMgoto $helpWindow $fragment
    return
  }
  HMreset_win $helpWindow
  update idletasks
  if {$fragment != ""} {
    HMgoto $helpWindow $fragment
  }
  set HELP_Info($helpWindow-Current) $file
  HMparse_html [HELP_getHtml $file] [list HMrender $helpWindow]
  HMset_state $helpWindow -stop 1
}

proc HELP_getHtml {file} {
  if {[catch {set fd [open $file]} msg]} {
    return "<HTML>
            <HEAD>
            <TITLE>Bad file \"$file\"</TITLE>
            </HEAD>
            <BODY>
            <H1>Error reading \"$file\"</H1><P>
            $msg
            </BODY>
            </HTML>"
  }
  set contents [read $fd]
  close $fd
  return $contents
}

proc HMlink_callback {helpWindow href} {
  global HELP_Info
  if {[string match #* $href]} {
    HELP_render $helpWindow $href
    return
  }
  if {[string match /* $href]} {
    set HELP_Info($helpWindow-Current) $href
  } else {
    set HELP_Info($helpWindow-Current) \
        [file dirname $HELP_Info($helpWindow-Current)]/$href
  }
  update
  HELP_render $helpWindow $HELP_Info($helpWindow-Current)
}

proc HMset_image {helpWindow handle src} {
  global HELP_Info
  if {[string match /* $src]} {
    set image $src
  } else {
    set image [file dirname $HELP_Info($helpWindow-Current)]/$src
  }
  update
  if {[string first " $image " " [image names] "] >= 0} {
    HMgot_image $handle $image
  } else {
    set type photo
    if {[file extension $image] == ".bmp"} {
      set type bitmap
    }
    catch {image create $type $image -file $image} image
    HMgot_image $handle $image
  }
}

proc HMtag_base {helpWindow param text} {
  global HELP_Info
  upvar #0 HM$helpWindow var
  HMextract_param $param href HELP_Info($helpWindow-Current)
}

#--------------------------------------------------------------#
# user-callable functions
#

proc HELP_help {dirName fileName} {
  global HELP_Info
  set helpWindow [HELP_newHelpWindow]
  set HELP_Info($helpWindow-Index) [file join $dirName index.html]
  set HELP_Info($helpWindow-Current) [file join $dirName $fileName]
  set HELP_Info($helpWindow-Stack) {}
  HMinit_win $helpWindow
  HELP_render $helpWindow $HELP_Info($helpWindow-Current)
}
