
#--------------------------------------------------------------#
# Oscilloscope for DSP simulator
#--------------------------------------------------------------#


#--------------------------------------------------------------#
# set constants
#


set SCOPE_Const(X_UNITS)            14
set SCOPE_Const(PP_X_UNIT)          30
set SCOPE_Const(Y_UNITS)            8                
set SCOPE_Const(PP_Y_UNIT)          30

set SCOPE_Const(MIN_X_POS)          0
set SCOPE_Const(MAX_X_POS)          10000
set SCOPE_Const(X_POS_DELTA_DELTA)  4
set SCOPE_Const(MIN_X_POS_DELTA)    -200
set SCOPE_Const(MAX_X_POS_DELTA)    200

set SCOPE_Const(MIN_Y_POS)          [expr -12 * $SCOPE_Const(PP_Y_UNIT)]
set SCOPE_Const(MAX_Y_POS)          [expr 12 * $SCOPE_Const(PP_Y_UNIT)]
set SCOPE_Const(Y_POS_DELTA_DELTA)  3
set SCOPE_Const(MIN_Y_POS_DELTA)    [expr -4 * $SCOPE_Const(PP_Y_UNIT)]
set SCOPE_Const(MAX_Y_POS_DELTA)    [expr 4 * $SCOPE_Const(PP_Y_UNIT)]


#--------------------------------------------------------------#
# normbutton implementation
#


proc normbutton {path cmd args} {
  set win [eval {button $path} $args]
  bind $win <ButtonPress-1> [list normbuttonPress $cmd]
  bind $win <ButtonRelease-1> {}
  return $win
}


proc normbuttonPress {cmd} {
  if {$cmd != ""} {
    uplevel #0 $cmd
  }
}


#--------------------------------------------------------------#
# pushbutton implementation
#


proc pushbutton {path varname initvalue onCmd offCmd args} {
  upvar #0 $varname var
  set win [eval {button $path} $args]
  bind $win <ButtonPress-1> [list pushbuttonPress $win $varname $onCmd]
  bind $win <ButtonRelease-1> [list pushbuttonRelease $win $varname $offCmd]
  if {$initvalue == 0} {
    set var 0
    $win config -relief raised
  } else {
    set var 1
    $win config -relief sunken
  }
  return $win
}


proc pushbuttonPress {win varname onCmd} {
  upvar #0 $varname var
  if {$var == 0} {
    # push button
    set var 2
    $win config -relief sunken
    if {$onCmd != ""} {
      uplevel #0 $onCmd
    }
  } else {
    # button will be released immediately
    set var 3
    $win config -relief raised
  }
}


proc pushbuttonRelease {win varname offCmd} {
  upvar #0 $varname var
  if {$var == 2} {
    # button has just been pushed
    set var 1
    $win config -relief sunken
  } else {
    # release button
    set var 0
    $win config -relief raised
    if {$offCmd != ""} {
      uplevel #0 $offCmd
    }
  }
}


#--------------------------------------------------------------#
# holdbutton implementation
#


proc holdbutton {path onCmd holdCmd msec offCmd args} {
  set win [eval {button $path} $args]
  bind $win <ButtonPress-1> [list holdbuttonPress $onCmd $holdCmd $msec]
  bind $win <ButtonRelease-1> [list holdbuttonRelease $offCmd]
  return $win
}


proc holdbuttonPress {onCmd holdCmd msec} {
  global tkPriv
  set tkPriv(holdbuttonTimer) \
    [after $msec [list holdbuttonHold $holdCmd $msec]]
  if {$onCmd != ""} {
    uplevel #0 $onCmd
  }
}


proc holdbuttonHold {holdCmd msec} {
  global tkPriv
  set tkPriv(holdbuttonTimer) \
    [after $msec [list holdbuttonHold $holdCmd $msec]]
  if {$holdCmd != ""} {
    uplevel #0 $holdCmd
  }
}


proc holdbuttonRelease {offCmd} {
  global tkPriv
  after cancel $tkPriv(holdbuttonTimer)
  if {$offCmd != ""} {
    uplevel #0 $offCmd
  }
}


#--------------------------------------------------------------#
# power on
#


proc SCOPE_PowerOn {lampCanvas lampId} {
  $lampCanvas itemconfigure $lampId -fill green
  update idletasks
}


proc SCOPE_PowerOff {lampCanvas lampId} {
  $lampCanvas itemconfigure $lampId -fill red
  update idletasks
  after 500 [list SCOPE_WillClose $lampCanvas]
}


proc SCOPE_WillClose {lampCanvas} {
  global SCOPE_State
  if {$SCOPE_State($lampCanvas,PowerOn) == 0} {
    set toplevelWindow [winfo toplevel $lampCanvas]
    SCOPE_Close $toplevelWindow
  }
}


#--------------------------------------------------------------#
# x position
#


proc SCOPE_FormatXPos {value} {
  return [format "%d s" $value]
}


proc SCOPE_XPosInit {displayCanvas displayId} {
  global SCOPE_State
  set SCOPE_State($displayCanvas,XPos) 0
  set value $SCOPE_State($displayCanvas,XPos)
  $displayCanvas itemconfigure $displayId \
    -text [SCOPE_FormatXPos $value]
  update idletasks
}


proc SCOPE_XPosPress {direction displayCanvas displayId} {
  global SCOPE_State
  if {$direction == "left"} {
    set SCOPE_State($displayCanvas,XPosDelta) -1
  } else {
    set SCOPE_State($displayCanvas,XPosDelta) 1
  }
  SCOPE_XPosHold $direction $displayCanvas $displayId
}


proc SCOPE_XPosHold {direction displayCanvas displayId} {
  global SCOPE_State
  global SCOPE_Const
  incr SCOPE_State($displayCanvas,XPos) \
       $SCOPE_State($displayCanvas,XPosDelta)
  if {$SCOPE_State($displayCanvas,XPos) < $SCOPE_Const(MIN_X_POS)} {
    set SCOPE_State($displayCanvas,XPos) $SCOPE_Const(MIN_X_POS)
  }
  if {$SCOPE_State($displayCanvas,XPos) > $SCOPE_Const(MAX_X_POS)} {
    set SCOPE_State($displayCanvas,XPos) $SCOPE_Const(MAX_X_POS)
  }
  if {$direction == "left"} {
    incr SCOPE_State($displayCanvas,XPosDelta) \
         -$SCOPE_Const(X_POS_DELTA_DELTA)
    if {$SCOPE_State($displayCanvas,XPosDelta) < \
        $SCOPE_Const(MIN_X_POS_DELTA)} {
      set SCOPE_State($displayCanvas,XPosDelta) \
          $SCOPE_Const(MIN_X_POS_DELTA)
    }
  } else {
    incr SCOPE_State($displayCanvas,XPosDelta) \
         $SCOPE_Const(X_POS_DELTA_DELTA)
    if {$SCOPE_State($displayCanvas,XPosDelta) > \
        $SCOPE_Const(MAX_X_POS_DELTA)} {
      set SCOPE_State($displayCanvas,XPosDelta) \
          $SCOPE_Const(MAX_X_POS_DELTA)
    }
  }
  set value $SCOPE_State($displayCanvas,XPos)
  $displayCanvas itemconfigure $displayId \
    -text [SCOPE_FormatXPos $value]
  update idletasks
}


proc SCOPE_XPosRelease {direction displayCanvas displayId} {
  global SCOPE_State
  [winfo parent $displayCanvas].screen configure \
    -xpos $SCOPE_State($displayCanvas,XPos)
}


#--------------------------------------------------------------#
# x coefficient
#


set SCOPE_State(XCoeffList) \
    [list   1.0   1.5   2.0   3.5   5.0   7.5 \
           10.0  15.0  20.0  35.0  50.0  75.0 \
          100.0 150.0 200.0 350.0 500.0 750.0]


proc SCOPE_FormatXCoeff {value} {
  if {$value < 1e1} {
    return [format "%3.1f s/cm" $value]
  }
  if {$value < 1e2} {
    return [format "%3.0f s/cm" $value]
  }
  if {$value < 1e3} {
    return [format "%3.0f s/cm" $value]
  }
}


proc SCOPE_XCoeffInit {displayCanvas displayId} {
  global SCOPE_State
  set SCOPE_State($displayCanvas,XCoeffIndex) 6
  set SCOPE_State($displayCanvas,XCoeff) \
    [lindex $SCOPE_State(XCoeffList) \
            $SCOPE_State($displayCanvas,XCoeffIndex)]
  set value $SCOPE_State($displayCanvas,XCoeff)
  $displayCanvas itemconfigure $displayId \
    -text [SCOPE_FormatXCoeff $value]
  update idletasks
}


proc SCOPE_XCoeffPlus {displayCanvas displayId} {
  global SCOPE_State
  if {$SCOPE_State($displayCanvas,XCoeffIndex) < 17} {
    incr SCOPE_State($displayCanvas,XCoeffIndex)
    set SCOPE_State($displayCanvas,XCoeff) \
      [lindex $SCOPE_State(XCoeffList) \
              $SCOPE_State($displayCanvas,XCoeffIndex)]
    set value $SCOPE_State($displayCanvas,XCoeff)
    $displayCanvas itemconfigure $displayId \
      -text [SCOPE_FormatXCoeff $value]
    update idletasks
    [winfo parent $displayCanvas].screen configure \
      -xcoeff $SCOPE_State($displayCanvas,XCoeff)
  }
}


proc SCOPE_XCoeffMinus {displayCanvas displayId} {
  global SCOPE_State
  if {$SCOPE_State($displayCanvas,XCoeffIndex) > 0} {
    incr SCOPE_State($displayCanvas,XCoeffIndex) -1
    set SCOPE_State($displayCanvas,XCoeff) \
      [lindex $SCOPE_State(XCoeffList) \
              $SCOPE_State($displayCanvas,XCoeffIndex)]
    set value $SCOPE_State($displayCanvas,XCoeff)
    $displayCanvas itemconfigure $displayId \
      -text [SCOPE_FormatXCoeff $value]
    update idletasks
    [winfo parent $displayCanvas].screen configure \
      -xcoeff $SCOPE_State($displayCanvas,XCoeff)
  }
}


#--------------------------------------------------------------#
# timebase section
#


proc SCOPE_Timebase {win} {
  set timebase [canvas $win -width 126 -height 240 \
                -background lightgray]

  set lamp_PowerOn [$timebase create oval 28 25 38 35 \
                    -outline darkgray -fill green]
  set button_PowerOn [pushbutton $timebase.button_PowerOn \
                      SCOPE_State($timebase,PowerOn) 1 \
                      [list SCOPE_PowerOn $timebase $lamp_PowerOn] \
                      [list SCOPE_PowerOff $timebase $lamp_PowerOn] \
                      -width 3 -height 1 -text "On"]
  place $button_PowerOn -x 70 -y 17

  $timebase create rectangle 10 68 116 93 -outline darkgray
  set display_XPos [$timebase create text 63 81 \
                    -fill blue -text "unknown"]
  set button_left [holdbutton $timebase.button_left \
                   [list SCOPE_XPosPress left $timebase $display_XPos] \
                   [list SCOPE_XPosHold left $timebase $display_XPos] \
                   250 \
                   [list SCOPE_XPosRelease left $timebase $display_XPos] \
                   -width 3 -height 1 -text "<-"]
  place $button_left -x 10 -y 107
  set button_right [holdbutton $timebase.button_right \
                    [list SCOPE_XPosPress right $timebase $display_XPos] \
                    [list SCOPE_XPosHold right $timebase $display_XPos] \
                    250 \
                    [list SCOPE_XPosRelease right $timebase $display_XPos] \
                    -width 3 -height 1 -text "->"]
  place $button_right -x 70 -y 107
  SCOPE_XPosInit $timebase $display_XPos

  $timebase create rectangle 10 158 116 183 -outline darkgray
  set display_XCoeff [$timebase create text 63 171 \
                      -fill blue -text "unknown"]
  set button_plus [normbutton $timebase.button_plus \
                   [list SCOPE_XCoeffPlus $timebase $display_XCoeff] \
                   -width 3 -height 1 -text "+"]
  place $button_plus -x 10 -y 197
  set button_minus [normbutton $timebase.button_minus \
                    [list SCOPE_XCoeffMinus $timebase $display_XCoeff] \
                    -width 3 -height 1 -text "-"]
  place $button_minus -x 70 -y 197
  SCOPE_XCoeffInit $timebase $display_XCoeff

  return $timebase
}


#--------------------------------------------------------------#
# y interpolation
#


proc SCOPE_IntOn {channelCanvas} {
  [winfo parent $channelCanvas].screen configure \
    -interpolation[string range $channelCanvas end end] 1
}


proc SCOPE_IntOff {channelCanvas} {
  [winfo parent $channelCanvas].screen configure \
    -interpolation[string range $channelCanvas end end] 0
}


proc SCOPE_LowOn {channelCanvas} {
  [winfo parent $channelCanvas].screen configure \
    -lowpassfilter[string range $channelCanvas end end] 1
}


proc SCOPE_LowOff {channelCanvas} {
  [winfo parent $channelCanvas].screen configure \
    -lowpassfilter[string range $channelCanvas end end] 0
}


#--------------------------------------------------------------#
# y overdrive indicators
#


proc SCOPE_OverdriveOff {channelCanvas} {
  $channelCanvas itemconfigure TopOverdrive -fill darkgray
  $channelCanvas itemconfigure BottomOverdrive -fill darkgray
}


proc SCOPE_TopOverdriveOn {channelCanvas} {
  $channelCanvas itemconfigure TopOverdrive -fill yellow
}


proc SCOPE_BottomOverdriveOn {channelCanvas} {
  $channelCanvas itemconfigure BottomOverdrive -fill yellow
}


#--------------------------------------------------------------#
# y position
#


proc SCOPE_YPosInit {channelCanvas} {
  global SCOPE_State
  global SCOPE_Const
  set channel [string range $channelCanvas end end]
  set numberSignals 4
  set SCOPE_State($channelCanvas,YPos) \
      [expr (1 + $channel * 2) * \
            (4 * $SCOPE_Const(PP_Y_UNIT)) / $numberSignals \
            - 4 * $SCOPE_Const(PP_Y_UNIT)]
}


proc SCOPE_YPosPress {direction channelCanvas} {
  global SCOPE_State
  if {$direction == "up"} {
    set SCOPE_State($channelCanvas,YPosDelta) -1
  } else {
    set SCOPE_State($channelCanvas,YPosDelta) 1
  }
  SCOPE_YPosHold $direction $channelCanvas
}


proc SCOPE_YPosHold {direction channelCanvas} {
  global SCOPE_State
  global SCOPE_Const
  incr SCOPE_State($channelCanvas,YPos) \
       $SCOPE_State($channelCanvas,YPosDelta)
  if {$SCOPE_State($channelCanvas,YPos) < $SCOPE_Const(MIN_Y_POS)} {
    set SCOPE_State($channelCanvas,YPos) $SCOPE_Const(MIN_Y_POS)
  }
  if {$SCOPE_State($channelCanvas,YPos) > $SCOPE_Const(MAX_Y_POS)} {
    set SCOPE_State($channelCanvas,YPos) $SCOPE_Const(MAX_Y_POS)
  }
  if {$direction == "up"} {
    incr SCOPE_State($channelCanvas,YPosDelta) \
         -$SCOPE_Const(Y_POS_DELTA_DELTA)
    if {$SCOPE_State($channelCanvas,YPosDelta) < \
        $SCOPE_Const(MIN_Y_POS_DELTA)} {
      set SCOPE_State($channelCanvas,YPosDelta) \
          $SCOPE_Const(MIN_Y_POS_DELTA)
    }
  } else {
    incr SCOPE_State($channelCanvas,YPosDelta) \
         $SCOPE_Const(Y_POS_DELTA_DELTA)
    if {$SCOPE_State($channelCanvas,YPosDelta) > \
        $SCOPE_Const(MAX_Y_POS_DELTA)} {
      set SCOPE_State($channelCanvas,YPosDelta) \
          $SCOPE_Const(MAX_Y_POS_DELTA)
    }
  }
}


proc SCOPE_YPosRelease {direction channelCanvas} {
  global SCOPE_State
  [winfo parent $channelCanvas].screen configure \
    -ypos[string range $channelCanvas end end] \
    $SCOPE_State($channelCanvas,YPos)
}


#--------------------------------------------------------------#
# y coefficient
#


set SCOPE_State(YCoeffList) \
    [list   0.001   0.002   0.005 \
            0.010   0.020   0.050 \
            0.100   0.200   0.500 \
            1.000   2.000   5.000 \
           10.000  20.000  50.000 \
          100.000 200.000 500.000]


proc SCOPE_FormatYCoeff {value} {
  if {$value < 1e-2} {
    return [format "%3.0f mV/cm" [expr 1e3 * $value]]
  }
  if {$value < 1e-1} {
    return [format "%3.0f mV/cm" [expr 1e3 * $value]]
  }
  if {$value < 1.0} {
    return [format "%3.0f mV/cm" [expr 1e3 * $value]]
  }
  if {$value < 1e1} {
    return [format "%3.0f V/cm" $value]
  }
  if {$value < 1e2} {
    return [format "%3.0f V/cm" $value]
  }
  if {$value < 1e3} {
    return [format "%3.0f V/cm" $value]
  }
}


proc SCOPE_YCoeffInit {displayCanvas displayId} {
  global SCOPE_State
  set SCOPE_State($displayCanvas,YCoeffIndex) 9
  set SCOPE_State($displayCanvas,YCoeff) \
    [lindex $SCOPE_State(YCoeffList) \
            $SCOPE_State($displayCanvas,YCoeffIndex)]
  set value $SCOPE_State($displayCanvas,YCoeff)
  $displayCanvas itemconfigure $displayId \
    -text [SCOPE_FormatYCoeff $value]
  update idletasks
}


proc SCOPE_YCoeffPlus {displayCanvas displayId} {
  global SCOPE_State
  if {$SCOPE_State($displayCanvas,YCoeffIndex) < 17} {
    incr SCOPE_State($displayCanvas,YCoeffIndex)
    set SCOPE_State($displayCanvas,YCoeff) \
      [lindex $SCOPE_State(YCoeffList) \
              $SCOPE_State($displayCanvas,YCoeffIndex)]
    set value $SCOPE_State($displayCanvas,YCoeff)
    $displayCanvas itemconfigure $displayId \
      -text [SCOPE_FormatYCoeff $value]
    update idletasks
    [winfo parent $displayCanvas].screen configure \
      -ycoeff[string range $displayCanvas end end] \
      $SCOPE_State($displayCanvas,YCoeff)
  }
}


proc SCOPE_YCoeffMinus {displayCanvas displayId} {
  global SCOPE_State
  if {$SCOPE_State($displayCanvas,YCoeffIndex) > 0} {
    incr SCOPE_State($displayCanvas,YCoeffIndex) -1
    set SCOPE_State($displayCanvas,YCoeff) \
      [lindex $SCOPE_State(YCoeffList) \
              $SCOPE_State($displayCanvas,YCoeffIndex)]
    set value $SCOPE_State($displayCanvas,YCoeff)
    $displayCanvas itemconfigure $displayId \
      -text [SCOPE_FormatYCoeff $value]
    update idletasks
    [winfo parent $displayCanvas].screen configure \
      -ycoeff[string range $displayCanvas end end] \
      $SCOPE_State($displayCanvas,YCoeff)
  }
}


#--------------------------------------------------------------#
# channel section
#


proc SCOPE_Channel {win name} {
  set channel [canvas $win -width 126 -height 170 \
               -background lightgray]

  $channel create text 63 10 -fill brown -text $name

  set button_Int [pushbutton $channel.button_Int \
                  SCOPE_State($channel,Int) 0 \
                  [list SCOPE_IntOn $channel] \
                  [list SCOPE_IntOff $channel] \
                  -width 3 -height 1 -text "Int"]
  place $button_Int -x 0 -y 25
  set button_Low [pushbutton $channel.button_Low \
                  SCOPE_State($channel,Low) 0 \
                  [list SCOPE_LowOn $channel] \
                  [list SCOPE_LowOff $channel] \
                  -width 3 -height 1 -text "Low"]
  place $button_Low -x 0 -y 61

  $channel create oval 58 33 68 43 -tag TopOverdrive \
                       -outline darkgray -fill darkgray
  $channel create oval 58 69 68 79 -tag BottomOverdrive \
                       -outline darkgray -fill darkgray
  set button_up [holdbutton $channel.button_up \
                 [list SCOPE_YPosPress up $channel] \
                 [list SCOPE_YPosHold up $channel] \
                 250 \
                 [list SCOPE_YPosRelease up $channel] \
                 -width 3 -height 1 -text "up"]
  place $button_up -x 80 -y 25
  set button_dn [holdbutton $channel.button_dn \
                 [list SCOPE_YPosPress down $channel] \
                 [list SCOPE_YPosHold down $channel] \
                 250 \
                 [list SCOPE_YPosRelease down $channel] \
                 -width 3 -height 1 -text "dn"]
  place $button_dn -x 80 -y 61
  SCOPE_YPosInit $channel

  $channel create rectangle 10 105 116 130 -outline darkgray
  set display_YCoeff [$channel create text 63 118 \
                      -fill blue -text "unknown"]
  set button_plus [normbutton $channel.button_plus \
                   [list SCOPE_YCoeffPlus $channel $display_YCoeff] \
                   -width 3 -height 1 -text "+"]
  place $button_plus -x 10 -y 144
  set button_minus [normbutton $channel.button_minus \
                    [list SCOPE_YCoeffMinus $channel $display_YCoeff] \
                    -width 3 -height 1 -text "-"]
  place $button_minus -x 70 -y 144
  SCOPE_YCoeffInit $channel $display_YCoeff

  return $channel
}


#--------------------------------------------------------------#
# a single oscilloscope
#


proc SCOPE_PruneSignalName {name} {
  set len [string length $name]
  if {$len > 15} {
    set firstPart [string range $name 0 5]
    set lastPart [string range $name [expr $len - 6] end]
    return "$firstPart...$lastPart"
  } else {
    return $name
  }
}


proc SCOPE_Open {scopeNumber \
                 signalName0 signalNumber0 \
                 signalName1 signalNumber1 \
                 signalName2 signalNumber2 \
                 signalName3 signalNumber3 \
                 dataFile numberCycles numberSignals} {
  global SCOPE_Const
  set scopeWindow [toplevel .scopeWindow$scopeNumber -background lightgray]
  wm title $scopeWindow "4-Channel Oscilloscope [expr $scopeNumber + 1]"
  wm geometry $scopeWindow 600x440
  wm minsize $scopeWindow 600 440
  wm maxsize $scopeWindow 600 440
  set scope [canvas $scopeWindow.scope -width 580 -height 420 \
             -background lightgray]
  place $scope -x 10 -y 10

  set timebase [SCOPE_Timebase $scope.timebase]
  place $timebase -x 452 -y 0

  set signalName0 [SCOPE_PruneSignalName $signalName0]
  set channel0 [SCOPE_Channel $scope.channel0 $signalName0]
  place $channel0 -x 2 -y 250
  set signalName1 [SCOPE_PruneSignalName $signalName1]
  set channel1 [SCOPE_Channel $scope.channel1 $signalName1]
  place $channel1 -x 152 -y 250
  set signalName2 [SCOPE_PruneSignalName $signalName2]
  set channel2 [SCOPE_Channel $scope.channel2 $signalName2]
  place $channel2 -x 302 -y 250
  set signalName3 [SCOPE_PruneSignalName $signalName3]
  set channel3 [SCOPE_Channel $scope.channel3 $signalName3]
  place $channel3 -x 452 -y 250

  $scope create line 140 270 140 420 -fill darkgray
  $scope create line 141 270 141 420 -fill darkgray
  $scope create line 290 270 290 420 -fill darkgray
  $scope create line 291 270 291 420 -fill darkgray
  $scope create line 440 270 440 420 -fill darkgray
  $scope create line 441 270 441 420 -fill darkgray

  set screen [screen $scope.screen \
              $signalNumber0 $signalNumber1 $signalNumber2 $signalNumber3 \
              $dataFile $numberCycles $numberSignals \
              -xunits $SCOPE_Const(X_UNITS) \
              -ppxunit $SCOPE_Const(PP_X_UNIT) \
              -yunits $SCOPE_Const(Y_UNITS) \
              -ppyunit $SCOPE_Const(PP_Y_UNIT) \
              -background darkgreen]
  place $screen -x 4 -y 0

  return $scopeWindow
}


#--------------------------------------------------------------#
# multiple oscilloscopes
#


set SCOPE_State(openScopes) {}


proc SCOPE_NumberOpenScopes {} {
  global SCOPE_State
  return [llength $SCOPE_State(openScopes)]
}


proc SCOPE_Close {toplevelWindow} {
  global SCOPE_State
  set index [lsearch -exact $SCOPE_State(openScopes) $toplevelWindow]
  if {$index != -1} {
    set SCOPE_State(openScopes) \
        [lreplace $SCOPE_State(openScopes) $index $index]
    destroy $toplevelWindow
  }
}


proc SCOPE_CloseAll {} {
  global SCOPE_State
  while {[SCOPE_NumberOpenScopes] != 0} {
    SCOPE_Close [lindex $SCOPE_State(openScopes) 0]
  }
}


proc SCOPE_OpenForFiles {ctrlFile dataFile} {
  global SCOPE_State
  SCOPE_CloseAll
  set ctrl [open $ctrlFile r]
  gets $ctrl numberCycles
  gets $ctrl numberSignals
  set numberScopes [expr ($numberSignals + 3) / 4]
  for {set scope 0} {$scope < $numberScopes} {incr scope} {
    for {set i 0} {$i < 4} {incr i} {
      set signalName$i [gets $ctrl]
      set signalNumber$i [expr $scope * 4 + $i]
    }
    lappend SCOPE_State(openScopes) \
      [SCOPE_Open $scope \
                  $signalName0 $signalNumber0 \
                  $signalName1 $signalNumber1 \
                  $signalName2 $signalNumber2 \
                  $signalName3 $signalNumber3 \
                  $dataFile $numberCycles $numberSignals]
  }
  close $ctrl
}
